'''
Created on 04-06-2012

@author: DarkGL

http://yuraj.ucoz.com/half-life-formats.pdf
'''

import struct

#Sprite type constants
VP_PARALLEL_UPRIGHT     =   0x00
FACING_UPRIGHT          =   0x01
VP_PARALLEL             =   0x02
ORIENTED                =   0x03
VP_PARALLEL_ORIENTED    =   0x04

#Sprite texture format
SPR_NORMAL      =   0x0
SPR_ADDITIVE    =   0x01
SPR_INDEXALPHA  =   0x02
SPR_ALPHTEST    =   0x03

class spriteClass :

    def __init__( self ) :
        self.fileID         =   0
        self.sprVersion     =   0
        self.sprType        =   0
        self.sprTexture     =   0
        self.boundingRad    =   0.0
        self.maxWidth       =   0
        self.maxHeight      =   0
        self.frameNumber    =   0
        self.beamLen        =   0.0
        self.syncType       =   0
        self.sizePalette    =   0
        self.paletteList    =   []
        self.frameList      =   []
        
        self._msgError       =   []
        self._msgErrorNum    =   0
    
    def getRGBList( self , iNumber ) :
        return self.paletteList[ iNumber ]
    
    
    def errorNum( self ) :
        return self._msgErrorNum
    
    def errorGet( self ) :
        if self._msgErrorNum > 0 :
            self._msgErrorNum    =   self._msgErrorNum - 1
            
            szString    =   self._msgError[ 0 ]
            
            del self._msgError[ 0 ]
            
            return szString
        else :
            return ""
    
    def _errorAdd( self , szError ) :
        self._msgErrorNum    =   self._msgErrorNum + 1
        
        self._msgError.append( szError )
    
        
    def fromFile( self , szPath ) :
        
        try :
            pFile   =   open( szPath , "rb" )
        except IOError :
            self._errorAdd( "No File" )
            
            return False
        
        self.fileID =   pFile.read( 4 )
           
        if self.fileID  !=  "IDSP" :
            
            spriteClass.__init__( self )
                
            self._errorAdd( "Not spr file" )
                
            return False
        
        self.sprVersion , =   struct.unpack( "i" , pFile.read( 4 ) )
        
        if self.sprVersion != 0x02 :
            
            spriteClass.__init__( self )
            
            self._errorAdd( "Unknown spr version" )
            
            return False
        
        self.sprType , =   struct.unpack( "i" , pFile.read( 4 ) )
        
        if ( self.sprType !=  VP_PARALLEL_UPRIGHT and self.sprType !=  FACING_UPRIGHT and
            self.sprType !=  VP_PARALLEL and self.sprType != ORIENTED and 
            self.sprType !=  VP_PARALLEL_ORIENTED ) :
            
                spriteClass.__init__( self )
                 
                self._errorAdd( "Unknown spr type" )
            
                return False
            
        self.sprTexture  , =   struct.unpack( "i" , pFile.read( 4 ) )
        
        if ( self.sprTexture !=  SPR_NORMAL and self.sprTexture !=  SPR_ADDITIVE and
            self.sprTexture !=  SPR_INDEXALPHA and self.sprTexture != SPR_ALPHTEST ) :
            
                spriteClass.__init__( self )
                
                self._errorAdd( "Unknown spr texture type" )
            
                return False
        
        
        self.boundingRad ,  =   struct.unpack( "i" , pFile.read( 4 ) )
        
        self.maxWidth ,     =   struct.unpack( "i" , pFile.read( 4 ) )
        self.maxHeight ,    =   struct.unpack( "i" , pFile.read( 4 ) )
        
        if self.maxWidth <= 0 or self.maxHeight <= 0 :
                
                spriteClass.__init__( self )
                
                self._errorAdd( "Width or Height <= 0" )
            
                return False
        
        self.frameNumber , =   struct.unpack( "i" , pFile.read( 4 ) )
        
        if self.frameNumber <=  0 :
            
            spriteClass.__init__( self )
            
            self._errorAdd( "frameNumber <= 0" )
            
            return False
        
        self.beamLen ,      =   struct.unpack( "i" , pFile.read( 4 ) )
        self.syncType ,     =   struct.unpack( "i" , pFile.read( 4 ) )
        
        self.sizePalette ,  =   struct.unpack( "h" , pFile.read( 2 ) )
        
        if self.sizePalette <= 0 or self.sizePalette > 256 :
            
            spriteClass.__init__( self )
            
            self._errorAdd( "palette to big or small" )
            
            return False
        
        i   =   0
        
        while i < self.sizePalette : 
            
            self.paletteList.append( [ ord( pFile.read( 1 ) ) , ord( pFile.read( 1 ) ) , ord( pFile.read( 1 ) ) ] )
            
            i += 1
        
        i   =   0
        
        while i < self.frameNumber :
            iTmp            =   frameClass()
            
            frameGroup ,    =   struct.unpack( "i" , pFile.read( 4 ) )
            frameOriginX ,  =   struct.unpack( "i" , pFile.read( 4 ) )
            frameOriginY ,  =   struct.unpack( "i" , pFile.read( 4 ) )
            frameWidth ,    =   struct.unpack( "i" , pFile.read( 4 ) )
            frameHeight ,   =   struct.unpack( "i" , pFile.read( 4 ) )
            
            iTmp.setParamets( self , frameGroup , frameOriginX , frameOriginY , frameWidth , frameHeight )
            iTmp.setData( pFile )
            
            self.frameList.append( iTmp )
            
            i += 1
        
        pFile.close()


class frameClass :
    
    def __init__( self ) :
        self.frameGroup     =   0
        self.frameOriginX   =   0
        self.frameOriginY   =   0
        self.frameWidth     =   0
        self.frameHeight    =   0
        self.imageData      =   []
        self.rgbImageData   =   []
        
    def setParamets( self , parentSprite = 0 ,frameGroup = 0 , frameOriginX = 0 , frameOriginY = 0 , frameWidth = 0 , frameHeight = 0 ) :
        self.parentSprite   =   parentSprite
        self.frameGroup     =   frameGroup
        self.frameOriginX   =   frameOriginX
        self.frameOriginY   =   frameOriginY
        self.frameWidth     =   frameWidth
        self.frameHeight    =   frameHeight
    
    def setData( self , pFile ) :
        
        i = 0
        
        while i < self.frameWidth * self.frameHeight :

            iData   =   ord( pFile.read( 1 ) )
            
            self.imageData.append( iData )
            
            self.rgbImageData.append( self.parentSprite.getRGBList( iData ) )
            
            i = i + 1


class fileManager :
    
    from PIL import Image
    
    def loadImage( self , szPath ) :
        
        try :
            im  =   Image.open( szPath )
        except IOError :    
            return []
        
        return im.tostring()    
    
    def saveImage( self , iList , szPath ) :
        
        if is
        
        

        